# -*- coding: utf-8 -*-

__all__ = [
    'UserGroupService', 'PermissionService', 'UserGroupMemberService',
    'UserGroupPermissionService', 'VerificationService',
]

from . import user_group as UserGroupService
from . import permission as PermissionService
from . import user_group_member as UserGroupMemberService
from . import user_permission as UserPermissionService
from . import user_group_permission as UserGroupPermissionService
from . import verification as VerificationService



# -*- coding: utf-8 -*-

from sqlalchemy.exc import IntegrityError
from ..core import db
from ..models import UserGroupMember

def create(user_id, user_group_id):
    member = UserGroupMember(
        user_id=user_id,
        user_group_id=user_group_id,
    )
    db.session.add(member)
    try:
        db.session.commit()
    except:
        db.session.rollback()
        member = UserGroupMember.query.filter_by(
            user_id=user_id,
            user_group_id=user_group_id,
        ).first()
    return member

def get(id):
    return UserGroupMember.query.get(id)

def delete(id):
    member = UserGroupMember.query.get(id)
    if member:
        db.session.delete(member)
    db.session.commit()

def get_users_by_group(user_group_id):
    rows = UserGroupMember.query.filter_by(
        user_group_id=user_group_id
    ).with_entities(
        UserGroupMember.user_id
    ).all()
    return [row.user_id for row in rows]

def get_user_groups_by_user(user_id):
    rows = UserGroupMember.query.filter_by(
        user_id=user_id
    ).with_entities(
        UserGroupMember.user_group_id
    ).all()
    return [row.user_group_id for row in rows]

def filter_user_group_members(filter_by, offset, limit, sort_field='created_at', sort_dir='desc'):
    query = UserGroupMember.query
    if filter_by:
        query = query.filter_by(**filter_by)
    field = getattr(UserGroupMember, sort_field)
    order_by = getattr(field, sort_dir.lower())()
    return query.order_by(order_by).offset(offset).limit(limit).all()

def rest(obj):
    return dict(
        id=obj.id,
        user_id=obj.user_id,
        user_group_id=obj.user_group_id,
    )



# -*- coding: utf-8 -*-

from datetime import datetime

from .core import db

class Permission(db.Model):

    __tablename__ = 'permission'
    __table_args__ = (
        db.UniqueConstraint('code', name='ux_permission_code'),
    )

    id = db.Column(db.Integer, primary_key=True)
    title = db.Column(db.String(64), default='', nullable=False)
    code = db.Column(db.String(64))
    created_at = db.Column(db.DateTime, default=datetime.utcnow, nullable=False)

    def __str__(self):
        return '<Permission id={.id} title={.title} code={.code}>'.format(self)

class UserGroup(db.Model):

    __tablename__ = 'user_group'

    id = db.Column(db.Integer, primary_key=True)
    title = db.Column(db.String(64), default='', nullable=False)
    code = db.Column(db.String(64))
    created_at = db.Column(db.DateTime, default=datetime.utcnow, nullable=False)

    def __str__(self):
        return '<UserGroup id={.id} title={.title} code={.code}>'.format(self)

class UserGroupMember(db.Model):

    __tablename__ = 'user_group_member'

    __table_args__ = (
        db.UniqueConstraint('user_id', 'user_group_id', name='ux_user_in_user_group'),
    )

    id = db.Column(db.Integer, primary_key=True)
    user_id = db.Column(db.Integer, nullable=False)
    user_group_id = db.Column(db.Integer, nullable=False)
    created_at = db.Column(db.DateTime, default=datetime.utcnow, nullable=False)

    def __str__(self):
        return '<UserGroupMember id={.id} user_id={.user_id} user_group_id={.user_group_id}>'.format(self)

class UserPermission(db.Model):

    __tablename__ = 'user_permission'
    __table_args__ = (
        db.UniqueConstraint('user_id', 'permission_id', name='ux_user_permission'),
    )

    id = db.Column(db.Integer, primary_key=True)
    user_id = db.Column(db.Integer, nullable=False)
    permission_id = db.Column(db.Integer, nullable=False)
    created_at = db.Column(db.DateTime, default=datetime.utcnow, nullable=False)

    def __str__(self):
        return '<UserPermission id={.id} user_id={.user_id} permission_id={.permission_id}>'.format(self)

class UserGroupPermission(db.Model):

    __tablename__ = 'user_group_permission'
    __table_args__ = (
        db.UniqueConstraint('user_group_id', 'permission_id', name='ux_user_permission'),
    )

    id = db.Column(db.Integer, primary_key=True)
    user_group_id = db.Column(db.Integer, nullable=False)
    permission_id = db.Column(db.Integer, nullable=False)
    created_at = db.Column(db.DateTime, default=datetime.utcnow, nullable=False)

    def __str__(self):
        return '<UserGroupPermission id={.id} user_id={.user_id} permission_id={.permission_id}>'.format(self)



# -*- coding: utf-8 -*-

from sqlalchemy.exc import IntegrityError

from ..core import db
from ..models import UserGroupPermission

def create(user_group_id, permission_id):
    user_group_permission = UserGroupPermission(
        user_group_id=user_group_id,
        permission_id=permission_id,
    )
    db.session.add(user_group_permission)
    try:
        db.session.commit()
    except IntegrityError:
        db.session.rollback()
        user_group_permission = UserGroupPermission.query.filter_by(
            user_group_id=user_group_id,
            permission_id=permission_id,
        ).first()
    return user_group_permission

def get(id):
    return UserGroupPermission.query.get(id)

def delete(id):
    user_group_permission = UserGroupPermission.query.get(id)
    if user_group_permission:
        db.session.delete(user_group_permission)
    db.session.commit()

def delete_by_permission(permission_id):
    user_group_permissions = UserGroupPermission.query.filter_by(
        permission_id=permission_id
    ).all()
    for user_group_permission in user_group_permissions:
        db.session.delete(user_group_permission)
    db.session.commit()

def delete_by_user_group(user_group_id):
    user_group_permissions = UserGroupPermission.query.filter_by(
        user_group_id=user_group_id,
    ).all()
    for user_group_permission in user_group_permissions:
        db.session.delete(user_group_permission)
    db.session.commit()

def get_user_groups_by_permission(permission_id):
    rows = UserGroupPermission.query.filter_by(
        permission_id=permission_id
    ).with_entities(
        UserGroupPermission.user_group_id
    ).all()
    return [row.user_group_id for row in rows]

def get_permissions_by_user_group(user_group_id):
    rows = UserGroupPermission.query.filter_by(
        user_group_id=user_group_id
    ).with_entities(
        UserGroupPermission.permission_id
    ).all()
    return [row.permission_id for row in rows]

def filter_user_group_permissions(filter_by, offset, limit, sort_field='created_at', sort_dir='desc'):
    query = UserGroupPermission.query
    if filter_by:
        query = query.filter_by(**filter_by)
    field = getattr(UserGroupPermission, sort_field)
    order_by = getattr(field, sort_dir.lower())()
    return query.order_by(order_by).offset(offset).limit(limit).all()

def rest(user_permission):
    return dict(
        id=user_permission.id,
        user_group_id=user_permission.user_group_id,
        permission_id=user_permission.permission_id,
    )



# -*- coding: utf-8 -*-

from ..core import db
from ..models import UserGroup

def create(title, code=None):
    user_group = UserGroup(
        title=title,
        code=code,
    )
    db.session.add(user_group)
    db.session.commit()
    return user_group

def delete(id):
    user_group = UserGroup.query.get(id)
    if user_group:
        db.session.delete(user_group)
    db.session.commit()

def rename(id, title):
    user_group = UserGroup.query.get(id)
    user_group.title = title
    db.session.add(user_group)
    db.session.commit()
    return user_group

def update_code(id, code):
    user_group = UserGroup.query.get(id)
    user_group.code = code
    db.session.add(user_group)
    db.session.commit()
    return user_group

def get_user_groups():
    return UserGroup.query.all()

def get_user_groups_by_ids(ids):
    return UserGroup.query.filter(UserGroup.id.in_(ids)).all()

def rest(user_group):
    return dict(
        id=user_group.id,
        title=user_group.title,
        code=user_group.code,
    )

def get(id):
    return UserGroup.query.get(id)

def filter_user_groups(filter_by, offset, limit, sort_field='created_at', sort_dir='desc'):
    query = UserGroup.query
    if filter_by:
        query = query.filter_by(**filter_by)
    field = getattr(UserGroup, sort_field)
    order_by = getattr(field, sort_dir.lower())()
    return query.order_by(order_by).offset(offset).limit(limit).all()



# -*- coding: utf-8 -*-

from sqlalchemy.exc import IntegrityError

from ..core import db
from ..models import UserPermission

def create(user_id, permission_id):
    user_permission = UserPermission(
        user_id=user_id,
        permission_id=permission_id,
    )
    db.session.add(user_permission)
    try:
        db.session.commit()
    except IntegrityError:
        db.session.rollback()
        user_permission = UserPermission.query.filter_by(
            user_id=user_id,
            permission_id=permission_id,
        ).first()
    return user_permission

def delete(user_permission_id):
    user_permission = UserPermission.query.get(user_permission_id)
    if user_permission:
        db.session.delete(user_permission)
    db.session.commit()

def get(id):
    return UserPermission.query.get(id)

def delete_by_user(user_id):
    user_permissions = UserPermission.query.filter_by(
        user_id=user_id
    ).all()
    for user_permission in user_permissions:
        db.session.delete(user_permission)
    db.session.commit()

def delete_by_permission(permission_id):
    user_permissions = UserPermission.query.filter_by(
        permission_id=permission_id,
    ).all()
    for user_permission in user_permissions:
        db.session.delete(user_permission)
    db.session.commit()

def get_users_by_permission(permission_id):
    rows = UserPermission.query.filter_by(
        permission_id=permission_id
    ).with_entities(
        UserPermission.user_id
    ).all()
    return [row.user_id for row in rows]

def get_permissions_by_user(user_id):
    rows = UserPermission.query.filter_by(
        user_id=user_id
    ).with_entities(
        UserPermission.permission_id
    ).all()
    return [row.permission_id for row in rows]

def filter_user_permissions(filter_by, offset, limit, sort_field='created_at', sort_dir='desc'):
    query = UserPermission.query
    if filter_by:
        query = query.filter_by(**filter_by)
    field = getattr(UserPermission, sort_field)
    order_by = getattr(field, sort_dir.lower())()
    return query.order_by(order_by).offset(offset).limit(limit).all()

def rest(user_permission):
    return dict(
        id=user_permission.id,
        user_id=user_permission.user_id,
        permission_id=user_permission.permission_id,
    )



# -*- coding: utf-8 -*-

from ..core import db
from .user_permission import (
    get_permissions_by_user,
    delete_by_user as delete_user_permissions_by_user,
)
from .user_group_permission import get_permissions_by_user_group
from .user_group_member import get_user_groups_by_user

def get_user_group_permissions_by_user(user_id):
    user_permission_ids = set()
    user_group_ids = get_user_groups_by_user(user_id)
    for user_group_id in user_group_ids:
        user_group_permission_ids = set(get_permissions_by_user_group(user_group_id))
        user_permission_ids.update(user_group_permission_ids)
    return user_permission_ids

def get_user_permissions(user_id):
    user_permission_ids = set(get_permissions_by_user(user_id))
    user_group_permission_ids = get_user_group_permissions_by_user(user_id)
    return user_permission_ids | user_group_permission_ids

def has_permission(user_id, permission_id):
    return permission_id in get_user_permissions(user_id)



# -*- coding: utf-8 -*-

from flask import Blueprint, render_template, current_app, abort, url_for, redirect, request, flash
from flask_login import login_user, logout_user

bp = Blueprint('perm-admin', __name__, template_folder='templates', static_folder='static')

@bp.route('/')
def index():
    if not bp.perm.has_perm_admin_logined():
        return redirect(url_for('perm-admin.login'))

    render_data = {
        'base_api_url': current_app.config.get('PERM_ADMIN_API_PREFIX'),
        'base_web_url': current_app.config.get('PERM_ADMIN_PREFIX'),
        'debug': current_app.config.get('DEBUG'),
    }

    return render_template('/perm-admin/index.html', **render_data)

@bp.route('/login', methods=['GET', 'POST'])
def login():
    if request.method == 'POST':
        username = request.form['username']
        password = request.form['password']
        if bp.perm.check_perm_admin_auth(username, password):
            bp.perm.login_perm_admin()
            return redirect(url_for('perm-admin.index'))
        else:
            flash(u'Invalid Password', 'error')
            return redirect(url_for('perm-admin.login'))
    return render_template('/perm-admin/login.html')

@bp.route('/logout')
def logout():
    bp.perm.logout_perm_admin()
    return redirect(url_for('perm-admin.login'))



# -*- coding: utf-8 -*-

import logging
from functools import wraps
from flask import session, request
from werkzeug.security import generate_password_hash
from werkzeug.security import check_password_hash
from .core import db

class Perm(object):

    class Denied(Exception):
        pass

    def __init__(self, app=None):
        self.app = app
        self.user_callback = None
        self.current_user_callback = None
        self.users_callback = None
        self.users_count_callback = None
        self.admin_logger = logging.getLogger('flask_perm.admin')
        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        """
        Configuration Items:

        * PERM_DB
        * PERM_USERS_GETTER
        """
        db.app = app
        db.init_app(app)

        app.config.setdefault('PERM_ADMIN_API_PREFIX', '/perm-admin/api')
        app.config.setdefault('PERM_ADMIN_PREFIX', '/perm-admin')

        if app.config.get('PERM_ADMIN_ECHO'):
            self.admin_logger.setLevel(logging.INFO)
            self.admin_logger.addHandler(logging.StreamHandler())

        from . import models
        db.create_all()

        from .controllers import bp as api_bp
        api_bp.perm = self
        app.register_blueprint(api_bp, url_prefix=app.config.get('PERM_ADMIN_API_PREFIX'))

        from .admin import bp as admin_bp
        admin_bp.perm = self
        app.register_blueprint(admin_bp, url_prefix=app.config.get('PERM_ADMIN_PREFIX'))

    def log_admin_action(self, msg):
        if self.app.config.get('PERM_ADMIN_ECHO'):
            self.admin_logger.info(msg)

    def require_perm_admin(self, f):
        @wraps(f)
        def _(*args, **kwargs):
            if not session.get('perm_admin'):
                raise self.Denied
            return f(*args, **kwargs)
        return _

    def login_perm_admin(self):
        data = self.app.config.get('PERM_ADMIN_USERNAME') + '.' + \
               self.app.config.get('PERM_ADMIN_PASSWORD')
        session['perm_admin'] = generate_password_hash(data)

    def logout_perm_admin(self):
        session.pop('perm_admin', None)

    def has_perm_admin_logined(self):
        return self.check_perm_admin_session()

    def check_perm_admin_auth(self, username, password):
        return username == self.app.config.get('PERM_ADMIN_USERNAME') and \
            password == self.app.config.get('PERM_ADMIN_PASSWORD')

    def check_perm_admin_session(self):
        if request.authorization:
            auth = request.authorization
            return self.check_perm_admin_auth(auth.username, auth.password)
        perm_admin = session.get('perm_admin')
        if not perm_admin:
            return False
        data = self.app.config.get('PERM_ADMIN_USERNAME') + '.' + \
               self.app.config.get('PERM_ADMIN_PASSWORD')
        return check_password_hash(perm_admin, data)

    def user_loader(self, callback):
        self.user_callback = callback
        return callback

    def current_user_loader(self, callback):
        self.current_user_callback = callback
        return callback

    def users_loader(self, callback):
        self.users_callback = callback
        return callback

    def users_count_loader(self, callback):
        self.users_count_callback = callback
        return callback

    def load_user(self, user_id):
        if self.user_callback is None:
            raise NotImplementedError('You must register user_loader!')
        return self.user_callback(user_id)

    def load_current_user(self):
        if self.current_user_callback is None:
            raise NotImplementedError('You must register current_user_loader!')
        return self.current_user_callback()

    def load_users(self, filter_by={}, sort_field='created_at', sort_dir='desc', offset=0, limit=20):
        if self.users_callback is None:
            raise NotImplementedError('You must register users_loader!')
        return self.users_callback(**dict(
            filter_by=filter_by,
            sort_field=sort_field,
            sort_dir=sort_dir,
            offset=offset,
            limit=limit,
        ))

    def load_users_count(self):
        if self.users_count_callback is None:
            raise NotImplementedError('You must register users_count_loader')
        return self.users_count_callback()

    def has_permission(self, user_id, code):
        from .services import VerificationService, PermissionService
        permission = PermissionService.get_by_code(code)
        if not permission:
            return False
        return VerificationService.has_permission(user_id, permission.id)

    def get_user_permissions(self, user_id):
        from .services import VerificationService, PermissionService
        permission_ids = VerificationService.get_user_permissions(user_id)
        permissions = map(PermissionService.get, permission_ids)
        permissions = filter(None, permissions)
        permissions = map(PermissionService.rest, permissions)
        return permissions

    def get_all_permission_codes(self):
        from .services import PermissionService
        permissions = PermissionService.get_permissions()
        return [permission.code for permission in permissions]

    def require_permission(self, *codes):
        if self.current_user_callback is None:
            raise NotImplementedError('You must register current_user_loader!')

        def deco(func):
            @wraps(func)
            def _(*args, **kwargs):

                current_user = self.current_user_callback()
                current_user_id = current_user.id

                if not codes:
                    is_allowed = True
                if '*' in codes:
                    is_allowed = any(
                        self.has_permission(current_user_id, code)
                        for code in self.get_all_permission_codes()
                    )
                else:
                    is_allowed = any(self.has_permission(current_user_id, code) for code in codes)

                if is_allowed:
                    return func(*args, **kwargs)
                else:
                    raise self.__class__.Denied

            return _
        return deco



# -*- coding: utf-8 -*-

import logging
import json
from flask import Blueprint, request, jsonify, current_app
from sqlalchemy.exc import IntegrityError
from .core import db
from .services import (
    UserGroupService, PermissionService, UserGroupMemberService,
    UserPermissionService, UserGroupPermissionService,
    VerificationService,
)

bp = Blueprint('flask_perm_api', __name__)

def ok(data=None):
    return jsonify(code=0, message='success', data=data)

def bad_request(message='bad request', **data):
    return jsonify(code=1, message=message, data=data), 400

def not_found(message='not found', **data):
    return jsonify(code=1, message=message, data=data), 404

def forbidden(message='forbidden', **data):
    return jsonify(code=1, message=message, data=data), 403

def check_auth(username, password):
    return current_app.config['PERM_ADMIN_USERNAME'] == username and \
        current_app.config['PERM_ADMIN_PASSWORD'] == password

def log_action(data, **kwargs):
    data = dict(data)
    data.update(kwargs)
    bp.perm.log_admin_action(data)

@bp.before_request
def before_request():
    if not bp.perm.has_perm_admin_logined():
        return forbidden()

@bp.errorhandler(IntegrityError)
def detect_integrity_error(e):
    return bad_request('conflict')

@bp.route('/permissions', methods=['POST'])
def add_permission():
    data = request.get_json()
    if 'title' not in data:
        return bad_request('missing title field')
    if not data['title']:
        return bad_request('title is blank')
    title = data['title']
    code = data.get('code')
    permission = PermissionService.create(title, code)
    permission = PermissionService.rest(permission)
    log_action(permission, action='add', model='permission')
    return ok(permission)

def _get_filter_by():
    filter_by = request.args.get('_filters')
    if filter_by:
        try:
            filter_by = json.loads(filter_by)
        except ValueError:
            pass
    return filter_by

@bp.route('/permissions')
def get_permissions():
    offset = request.args.get('offset', type=int, default=0)
    limit = request.args.get('limit', type=int, default=20)
    sort_field = request.args.get('_sortField', 'created_at').lower()
    sort_dir = request.args.get('_sortDir', 'DESC').lower()
    filter_by = _get_filter_by()
    permissions = PermissionService.filter_permissions(
        filter_by, offset, limit, sort_field, sort_dir)
    permissions = map(PermissionService.rest, permissions)
    return ok(permissions)

@bp.route('/permissions/<int:permission_id>')
def get_permission(permission_id):
    permission = PermissionService.get(permission_id)
    if not permission:
        return not_found()
    permission = PermissionService.rest(permission)
    return ok(permission)

@bp.route('/permissions/<int:permission_id>', methods=['PUT'])
def update_permission(permission_id):
    permission = PermissionService.get(permission_id)
    if not permission:
        return not_found()
    if request.get_json().get('title'):
        PermissionService.rename(permission_id, request.get_json().get('title'))
    if request.get_json().get('code'):
        PermissionService.set_code(permission_id, request.get_json().get('code'))
    permission = PermissionService.rest(PermissionService.get(permission_id))
    log_action(permission, action='update', model='permission')
    return ok(permission)

@bp.route('/permissions/<int:permission_id>', methods=['DELETE'])
def delete_permission(permission_id):
    permission = PermissionService.get(permission_id)
    if not permission:
        return not_found()
    log_action(PermissionService.rest(permission), action='delete', model='permission')
    UserPermissionService.delete_by_permission(permission_id)
    UserGroupPermissionService.delete_by_permission(permission_id)
    PermissionService.delete(permission_id)
    return ok()

@bp.route('/user_permissions')
def get_user_permissions():
    offset = request.args.get('offset', type=int, default=0)
    limit = request.args.get('limit', type=int, default=20)
    sort_field = request.args.get('_sortField', 'created_at').lower()
    sort_dir = request.args.get('_sortDir', 'DESC').lower()
    filter_by = _get_filter_by()
    user_permissions = UserPermissionService.filter_user_permissions(
        filter_by, offset, limit, sort_field, sort_dir)
    user_permissions = map(UserPermissionService.rest, user_permissions)
    return ok(user_permissions)

@bp.route('/user_permissions', methods=['POST'])
def add_user_permission():
    data = request.get_json()
    try:
        permission_id = data['permission_id']
        user_id = data['user_id']
    except KeyError:
        return bad_request()

    permission = PermissionService.get(permission_id)
    if not permission:
        return not_found()
    user_permission = UserPermissionService.create(user_id, permission_id)
    user_permission = UserPermissionService.rest(user_permission)
    log_action(user_permission, action='add', model='user_permission')
    return ok(user_permission)

@bp.route('/user_permissions/<int:user_permission_id>', methods=['DELETE'])
def revoke_user_permission(user_permission_id):
    user_permission = UserPermissionService.get(user_permission_id)
    if not user_permission:
        return not_found()
    log_action(UserPermissionService.rest(user_permission), action='delete', model='user_permission')
    UserPermissionService.delete(user_permission_id)
    return ok()

@bp.route('/user_group_permissions')
def get_user_group_permissions():
    offset = request.args.get('offset', type=int, default=0)
    limit = request.args.get('limit', type=int, default=20)
    sort_field = request.args.get('_sortField', 'created_at').lower()
    sort_dir = request.args.get('_sortDir', 'DESC').lower()
    filter_by = _get_filter_by()
    user_group_permissions = UserGroupPermissionService.filter_user_group_permissions(
        filter_by, offset, limit, sort_field, sort_dir)
    user_group_permissions = map(UserGroupPermissionService.rest, user_group_permissions)
    return ok(user_group_permissions)

@bp.route('/user_group_permissions', methods=['POST'])
def add_user_group_permission():
    data = request.get_json()
    try:
        permission_id = data['permission_id']
        user_group_id = data['user_group_id']
    except KeyError:
        return bad_request()
    permission = PermissionService.get(permission_id)
    if not permission:
        return not_found()
    user_group_permission = UserGroupPermissionService.create(user_group_id, permission_id)
    user_group_permission = UserGroupPermissionService.rest(user_group_permission)
    log_action(user_group_permission, action='add', model='user_permission')
    return ok(user_group_permission)

@bp.route('/user_group_permissions/<int:user_group_permission_id>', methods=['DELETE'])
def revoke_user_group_permission(user_group_permission_id):
    user_group_permission = UserGroupPermissionService.get(user_group_permission_id)
    if not user_group_permission:
        return not_found()
    log_action(UserGroupPermissionService.rest(user_group_permission),
               action='delete', model='user_group_permission')
    UserGroupPermissionService.delete(user_group_permission_id)
    return ok()

@bp.route('/user_groups', methods=['POST'])
def add_user_group():
    try:
        data = request.get_json()
        title = data['title']
        code = data['code']
    except KeyError:
        return bad_request()
    user_group = UserGroupService.create(title, code)
    user_group = UserGroupService.rest(user_group)
    log_action(user_group, action='add', model='user_group')
    return ok(user_group)

@bp.route('/user_groups')
def get_user_groups():
    offset = request.args.get('offset', type=int, default=0)
    limit = request.args.get('limit', type=int, default=20)
    sort_field = request.args.get('_sortField', 'created_at').lower()
    sort_dir = request.args.get('_sortDir', 'DESC').lower()
    filter_by = _get_filter_by()
    user_groups = UserGroupService.filter_user_groups(
        filter_by, offset, limit, sort_field, sort_dir)
    user_groups = map(UserGroupService.rest, user_groups)
    return ok(user_groups)

@bp.route('/user_groups/<int:user_group_id>')
def get_user_group(user_group_id):
    user_group = UserGroupService.get(user_group_id)
    if not user_group:
        return not_found()
    user_group = UserGroupService.rest(user_group)
    return ok(user_group)

@bp.route('/user_groups/<int:user_group_id>', methods=['PUT'])
def update_user_group(user_group_id):
    user_group = UserGroupService.get(user_group_id)
    if not user_group:
        return not_found()
    data = request.get_json()
    if 'title' in data and data['title']:
        UserGroupService.rename(user_group_id, data['title'])
    if 'code' in data and data['code']:
        UserGroupService.update_code(user_group_id, data['code'])
    user_group = UserGroupService.rest(UserGroupService.get(user_group_id))
    log_action(user_group, action='update', model='user_group')
    return ok(user_group)

@bp.route('/user_groups/<int:user_group_id>', methods=['DELETE'])
def delete_user_group(user_group_id):
    user_group = UserGroupService.get(user_group_id)
    if not user_group:
        return not_found()
    log_action(UserGroupService.rest(user_group), action='delete', model='user_group')
    UserGroupPermissionService.delete_by_user_group(user_group_id)
    UserGroupService.delete(user_group_id)
    return ok()

@bp.route('/user_group_members')
def get_user_group_members():
    offset = request.args.get('offset', type=int, default=0)
    limit = request.args.get('limit', type=int, default=20)
    sort_field = request.args.get('_sortField', 'created_at').lower()
    sort_dir = request.args.get('_sortDir', 'DESC').lower()
    filter_by = _get_filter_by()
    members = UserGroupMemberService.filter_user_group_members(filter_by, offset, limit, sort_field, sort_dir)
    members = map(UserGroupMemberService.rest, members)
    return ok(members)

@bp.route('/user_group_members', methods=['POST'])
def add_user_group_member():
    data = request.get_json()
    try:
        user_id = data['user_id']
        user_group_id = data['user_group_id']
    except KeyError:
        return bad_request()
    user_group = UserGroupService.get(user_group_id)
    if not user_group:
        return not_found()
    member = UserGroupMemberService.create(user_id, user_group_id)
    member = UserGroupMemberService.rest(member)
    log_action(member, action='add', model='user_group_member')
    return ok(member)

@bp.route('/user_group_members/<int:user_group_member_id>', methods=['DELETE'])
def delete_user_from_user_group(user_group_member_id):
    user_group_member = UserGroupMemberService.get(user_group_member_id)
    if not user_group_member:
        return not_found()
    log_action(UserGroupMemberService.rest(user_group_member), action='delete', model='user_group_member')
    UserGroupMemberService.delete(user_group_member_id)
    return ok()

def jsonify_user(user):
    return dict(id=user.id, nickname=user.nickname)

@bp.route('/users')
def get_users():
    offset = request.args.get('offset', type=int, default=0)
    limit = request.args.get('limit', type=int, default=20)
    sort_field = request.args.get('_sortField', 'created_at').lower()
    sort_dir = request.args.get('_sortDir', 'DESC').lower()
    filter_by = _get_filter_by()
    users = bp.perm.load_users(filter_by, sort_field, sort_dir, offset, limit)
    users = map(jsonify_user, users)
    return ok(users)

@bp.route('/users/<int:user_id>')
def get_user(user_id):
    user = bp.perm.load_user(user_id)
    if not user:
        return not_found()
    return ok(jsonify_user(user))



"""
A permission flask extension inspired by Django.
"""

from setuptools import setup


setup(
    name='Flask-Perm',
    version='0.1.3',
    url='https://github.com/soasme/flask-perm',
    license='MIT',
    author='Ju Lin',
    author_email='soasme@gmail.com',
    description='Flask Permission Management Extension',
    long_description=__doc__,
    packages=['flask_perm'],
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    install_requires=[
        'Flask',
        'Flask-SQLAlchemy',
    ],
    classifiers=[
        'Framework :: Flask',
        'Natural Language :: English',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)



# -*- coding: utf-8 -*-

from ..core import db
from ..models import Permission

def create(title, code=None):
    permission = Permission(
        title=title,
        code=code
    )
    db.session.add(permission)
    db.session.commit()
    return permission

def delete(id):
    permission = Permission.query.get(id)
    if permission:
        db.session.delete(permission)
    db.session.commit()

def rename(id, title):
    permission = Permission.query.get(id)
    permission.title = title
    db.session.add(permission)
    db.session.commit()
    return permission

def rest(permission):
    return dict(
        id=permission.id,
        title=permission.title,
        code=permission.code
    )

def set_code(id, code):
    permission = Permission.query.get(id)
    permission.code = code
    db.session.add(permission)
    db.session.commit()
    return permission

def get(id):
    return Permission.query.get(id)

def get_permissions():
    return Permission.query.all()

def filter_permissions(filter_by, offset, limit, sort_field='created_at', sort_dir='desc'):
    query = Permission.query
    if filter_by:
        query = query.filter_by(**filter_by)
    field = getattr(Permission, sort_field)
    order_by = getattr(field, sort_dir.lower())()
    return query.order_by(order_by).offset(offset).limit(limit).all()

def get_permissions_by_ids(ids):
    return Permission.query.filter(Permission.id.in_(ids)).all()

def get_by_code(code):
    return Permission.query.filter_by(code=code).first()
